local AddonName, AddonTable = ...
local L = AddonTable.Localize
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")
if type(DuelResponseAddonConfig) ~= "table" then
	DuelResponseAddonConfig = {}
end
local parent
local RequesterNames = {}
local DefaultHigherLevelLimit = 5
local DefaultMinimumHealthRegainPercent = 20
local DefaultMinimumManaRegainPercent = 20
local DefaultMinimapPos = 45

if select(4, GetAddOnInfo("LibWho-2.0")) and not IsAddOnLoaded("LibWho-2.0") and IsAddOnLoadOnDemand("LibWho-2.0") then
	LoadAddOn("LibWho-2.0")
end
local wholib = LibStub:GetLibrary("LibWho-2.0"):Library()

-- Replace(string, find, replace, find, replace, find, replace, ...)
local function Replace(...)
	local s = ...
	for i = 2, (select("#", ...)), 2 do
		local f, r = select(i, ...)
		s = string.gsub(s, f, r or "")
	end
	return s
end

if select(4, GetAddOnInfo("ChatThrottleLib")) and not IsAddOnLoaded("ChatThrottleLib") and IsAddOnLoadOnDemand("ChatThrottleLib") then
	LoadAddOn("ChatThrottleLib")
end
if not ChatThrottleLib then print("|cFF00FFFF["..PlainAddonTitle.."]|r Install ChatThrottleLib!") end

local function Message(Text, ChatType, Target)
	local Text = Text
	if Text == nil then
		Text = ""
	elseif type(Text) ~= "string" then
		Text = tostring(Text)
	end
	local ChatType = tostring(ChatType):upper()
	if ChatType == "RAID" or ChatType == "GROUP" then
		if GetNumRaidMembers() > 0 then
			ChatType = "RAID"
		else
			ChatType = "PARTY"
		end
	end
	if ChatType == "PARTY" and GetNumPartyMembers() == 0 then
		ChatType = nil
	end
	if ChatType == "RAID" or ChatType == "PARTY" then
		ChatThrottleLib:SendChatMessage("BULK", AddonName, "|cFF00FFFF["..PlainAddonTitle.."]|r "..Text, ChatType)
	elseif ChatType == "CHANNEL" and Target and GetChannelName(Target) ~= 0 then
		ChatThrottleLib:SendChatMessage("BULK", AddonName, "|cFF00FFFF["..PlainAddonTitle.."]|r "..Text, ChatType, nil, GetChannelName(Target))
	elseif ChatType == "WHISPER" then
		ChatThrottleLib:SendChatMessage("ALERT", AddonName, "|cFF00FFFF["..PlainAddonTitle.."]|r "..Text, ChatType, nil, Target)
	else
		print("|cFF00FFFF["..PlainAddonTitle.."]|r "..Text)
	end
end

-- must call RegisterAddonMessagePrefix(AddonName) when the addon is loaded for this function to work
-- the messages are sent to the CHAT_MSG_ADDON event
local function AddonMessage(Text, ChatType, Target)
	ChatThrottleLib:SendAddonMessage("NORMAL", AddonName, Text, ChatType, Target)
end

local function Whisper(name, msg)
	if name and not DuelResponseAddonConfig.DisableWhispers then
		Message(msg, "WHISPER", name)
	end
end

local function Checks(name, diff)
	if DuelResponseAddonConfig.DeclineAllDuels then
		Whisper(name, L["I don't like to duel."])
		CancelDuel()
	elseif diff and not DuelResponseAddonConfig.DisableHigherLevelDecline and diff >= (DuelResponseAddonConfig.DeclineHigherLevelLimit or DefaultHigherLevelLimit) then
		Whisper(name, Replace(L["You are <NUMBER> levels higher than me, so it's fairly obvious that you'd win. Go and challenge someone else."], "<NUMBER>", diff))
		CancelDuel()
	elseif DuelResponseAddonConfig.PVPDecline and ( UnitIsPVP("player") or UnitIsPVPFreeForAll("player") ) and not UnitIsPVPSanctuary("player") then
		Whisper(name, L["I don't duel while my PvP flag is on."])
		CancelDuel()
	elseif not DuelResponseAddonConfig.DisableAFKDecline and UnitIsAFK("player") then
		Whisper(name, L["I am currently marked as Away."])
		SendChatMessage(L["Away."],"AFK")
		CancelDuel()
	elseif not DuelResponseAddonConfig.DisableDNDDecline and UnitIsDND("player") then
		Whisper(name, L["I am currently marked as Busy."])
		SendChatMessage(L["Busy."],"DND")
		CancelDuel()
	elseif not DuelResponseAddonConfig.DisableResSicknessDecline and UnitAura("player", (GetSpellInfo(15007--[[Resurrection Sickness]])), nil, "HARMFUL") then
		Whisper(name, L["I don't duel while I have Resurrection Sickness."])
		CancelDuel()
	elseif not DuelResponseAddonConfig.DisableLowHealthDecline and ( 100 - ( ( UnitHealth("player") / UnitHealthMax("player") ) * 100 ) ) >= (DuelResponseAddonConfig.DisableLowHealthDeclineLimit or DefaultMinimumHealthRegainPercent) then
		Whisper(name, L["Please allow me to heal first."])
		CancelDuel()
	elseif not DuelResponseAddonConfig.DisableLowManaDecline and UnitPowerMax("player", SPELL_POWER_MANA) > 0 and ( 100 - ( ( UnitPower("player", SPELL_POWER_MANA) / UnitPowerMax("player", SPELL_POWER_MANA) ) * 100 ) ) >= (DuelResponseAddonConfig.DisableLowManaDeclineLimit or DefaultMinimumManaRegainPercent) then
		Whisper(name, L["Please allow me to regain my mana first."])
		CancelDuel()
	end
end

function wholib:UserDataReturned(user, time, cached)
	local NAME = nil
	if cached then
		NAME = user.Name
	else
		if user and user.Name then
			NAME = user.Name
		else
			NAME = RequesterNames[1]
		end
		tremove(RequesterNames, 1)
	end
	if NAME then
		if user and tonumber(user.Level) then
			Message(L["Challenger"]..": "..NAME)
			Checks(NAME, tonumber(user.Level) - UnitLevel("player"))
		else
			Message(L["Enemy Challenger"]..": "..NAME)
			if not DuelResponseAddonConfig.DisableEnemyDecline then
				CancelDuel()
			else
				Checks()
			end
		end
	end
end

local USER_DATA_RETURNED = {callback = "UserDataReturned"}

local function OnEvent(self, event, arg1)
    if event == "DUEL_REQUESTED" then
		if IsControlKeyDown() or IsAltKeyDown() then
			Message(L["Challenger"]..": "..arg1)
			return
		end
		if DuelResponseAddonConfig.DisableWhispers and ( DuelResponseAddonConfig.DeclineAllDuels or ( DuelResponseAddonConfig.DisableHigherLevelDecline and DuelResponseAddonConfig.DisableEnemyDecline ) ) then
			Message(L["Challenger"]..": "..arg1)
			Checks()
		else
			local user = wholib:UserInfo(arg1, USER_DATA_RETURNED)
			if user then
				wholib:UserDataReturned(user, 0, 1)
			else
				tinsert(RequesterNames, arg1)
			end
		end
	elseif event == "ADDON_LOADED" then
		if arg1 == AddonName then
			if type(DuelResponseAddonConfig) ~= "table" then
				DuelResponseAddonConfig = {}
			end
			DuelResponse_MinimapButton_Reposition()
			if DuelResponseAddonConfig.DisableMinimapButton then
				DuelResponse_MinimapButton:Hide()
			end
			Message(L["Load Successful."])
		end
	end
end
local EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("ADDON_LOADED")
EventFrame:RegisterEvent("DUEL_REQUESTED")
EventFrame:SetScript("OnEvent", OnEvent)

local function SaveOptionsFrameSettings()
	if _G[parent.."DisableWhispers"]:GetChecked() then
		DuelResponseAddonConfig.DisableWhispers = nil
	else
		DuelResponseAddonConfig.DisableWhispers = true
	end
	if _G[parent.."DeclineAllDuels"]:GetChecked() then
		DuelResponseAddonConfig.DeclineAllDuels = true
	else
		DuelResponseAddonConfig.DeclineAllDuels = nil
	end
	if _G[parent.."PVPDecline"]:GetChecked() then
		DuelResponseAddonConfig.PVPDecline = true
	else
		DuelResponseAddonConfig.PVPDecline = nil
	end
	if UnitPowerMax("player", SPELL_POWER_MANA) == 0 or _G[parent.."DisableLowManaDecline"]:GetChecked() then
		DuelResponseAddonConfig.DisableLowManaDecline = nil
	else
		DuelResponseAddonConfig.DisableLowManaDecline = true
	end
	if UnitPowerMax("player", SPELL_POWER_MANA) == 0 or _G[parent.."DisableLowManaDeclineLimit"]:GetNumber() == DefaultMinimumManaRegainPercent then
		DuelResponseAddonConfig.DisableLowManaDeclineLimit = nil
	else
		DuelResponseAddonConfig.DisableLowManaDeclineLimit = _G[parent.."DisableLowManaDeclineLimit"]:GetNumber()
	end
	if _G[parent.."DisableLowHealthDecline"]:GetChecked() then
		DuelResponseAddonConfig.DisableLowHealthDecline = nil
	else
		DuelResponseAddonConfig.DisableLowHealthDecline = true
	end
	if _G[parent.."DisableLowHealthDeclineLimit"]:GetNumber() == DefaultMinimumHealthRegainPercent then
		DuelResponseAddonConfig.DisableLowHealthDeclineLimit = nil
	else
		DuelResponseAddonConfig.DisableLowHealthDeclineLimit = _G[parent.."DisableLowHealthDeclineLimit"]:GetNumber()
	end
	if _G[parent.."DisableDNDDecline"]:GetChecked() then
		DuelResponseAddonConfig.DisableDNDDecline = nil
	else
		DuelResponseAddonConfig.DisableDNDDecline = true
	end
	if _G[parent.."DisableAFKDecline"]:GetChecked() then
		DuelResponseAddonConfig.DisableAFKDecline = nil
	else
		DuelResponseAddonConfig.DisableAFKDecline = true
	end
	if _G[parent.."DisableResSicknessDecline"]:GetChecked() then
		DuelResponseAddonConfig.DisableResSicknessDecline = nil
	else
		DuelResponseAddonConfig.DisableResSicknessDecline = true
	end
	if _G[parent.."DisableEnemyDecline"]:GetChecked() then
		DuelResponseAddonConfig.DisableEnemyDecline = nil
	else
		DuelResponseAddonConfig.DisableEnemyDecline = true
	end
	if _G[parent.."DisableHigherLevelDecline"]:GetChecked() then
		DuelResponseAddonConfig.DisableHigherLevelDecline = nil
	else
		DuelResponseAddonConfig.DisableHigherLevelDecline = true
	end
	if _G[parent.."DeclineHigherLevelLimit"]:GetNumber() == DefaultHigherLevelLimit
	or _G[parent.."DeclineHigherLevelLimit"]:GetNumber() == 0 then
		DuelResponseAddonConfig.DeclineHigherLevelLimit = nil
	else
		DuelResponseAddonConfig.DeclineHigherLevelLimit = _G[parent.."DeclineHigherLevelLimit"]:GetNumber()
	end
	if _G[parent.."DisableMinimapButton"]:GetChecked() then
		DuelResponseAddonConfig.DisableMinimapButton = nil
		DuelResponse_MinimapButton:Show()
	else
		DuelResponseAddonConfig.DisableMinimapButton = true
		DuelResponse_MinimapButton:Hide()
	end
end

local function ResetDefaults()
	wipe(DuelResponseAddonConfig)
	DuelResponse_MinimapButton:Show()
	DuelResponse_MinimapButton_Reposition()
end

local function LoadOptionsFrameSettings()
	_G[parent.."DisableWhispers"]:SetChecked(not DuelResponseAddonConfig.DisableWhispers)
	_G[parent.."DeclineAllDuels"]:SetChecked(not not DuelResponseAddonConfig.DeclineAllDuels)
	_G[parent.."PVPDecline"]:SetChecked(not not DuelResponseAddonConfig.PVPDecline)
	if UnitPowerMax("player", SPELL_POWER_MANA) > 0 then
		_G[parent.."DisableLowManaDecline"]:SetChecked(not DuelResponseAddonConfig.DisableLowManaDecline)
		_G[parent.."DisableLowManaDeclineLimit"]:SetNumber(DuelResponseAddonConfig.DisableLowManaDeclineLimit or DefaultMinimumManaRegainPercent)
	else
		_G[parent.."DisableLowManaDecline"]:Hide()
		_G[parent.."DisableLowManaDeclineLimit"]:Hide()
		_G[parent.."DisableLowManaDeclineLimitLabel"]:Hide()
		_G[parent.."DisableLowManaDeclineLimitPercentSign"]:Hide()
	end
	_G[parent.."DisableLowHealthDecline"]:SetChecked(not DuelResponseAddonConfig.DisableLowHealthDecline)
	_G[parent.."DisableLowHealthDeclineLimit"]:SetNumber(DuelResponseAddonConfig.DisableLowHealthDeclineLimit or DefaultMinimumHealthRegainPercent)
	_G[parent.."DisableDNDDecline"]:SetChecked(not DuelResponseAddonConfig.DisableDNDDecline)
	_G[parent.."DisableAFKDecline"]:SetChecked(not DuelResponseAddonConfig.DisableAFKDecline)
	_G[parent.."DisableResSicknessDecline"]:SetChecked(not DuelResponseAddonConfig.DisableResSicknessDecline)
	_G[parent.."DisableEnemyDecline"]:SetChecked(not DuelResponseAddonConfig.DisableEnemyDecline)
	_G[parent.."DisableHigherLevelDecline"]:SetChecked(not DuelResponseAddonConfig.DisableHigherLevelDecline)
	_G[parent.."DeclineHigherLevelLimit"]:SetNumber(DuelResponseAddonConfig.DeclineHigherLevelLimit or DefaultHigherLevelLimit)
	_G[parent.."DisableMinimapButton"]:SetChecked(not DuelResponseAddonConfig.DisableMinimapButton)
end

local function LocalizeFontStrings(frame)
	for _, f in next, {frame:GetChildren()} do
		if f:GetObjectType() == "FontString" then
			f:SetText(L[f:GetText()])
		else
			for _, f in next, {f:GetRegions()} do
				if f:GetObjectType() == "FontString" then
					f:SetText(L[f:GetText()])
				end
			end
		end
	end
end

function DuelResponseAddonOptionsFrame_OnLoad(self)
	parent = self:GetName()
	LocalizeFontStrings(self)
	_G[parent.."TitleString"]:SetText(L["DuelResponse Config"].." "..GetAddOnMetadata(AddonName, "Version"))
	self.name = PlainAddonTitle
	self.okay = SaveOptionsFrameSettings
	self.default = ResetDefaults
	self.refresh = LoadOptionsFrameSettings
	InterfaceOptions_AddCategory(self)
end

function SlashCmdList.DuelResponseAddonSlashHandler()
	InterfaceOptionsFrame_OpenToCategory(_G[parent])
end
SLASH_DuelResponseAddonSlashHandler1 = "/"..PlainAddonTitle:lower():gsub("%s", "")
SLASH_DuelResponseAddonSlashHandler2 = "/duelresponse"
SLASH_DuelResponseAddonSlashHandler3 = "/duelresp"
SLASH_DuelResponseAddonSlashHandler4 = "/duelr"
SLASH_DuelResponseAddonSlashHandler5 = "/dr"

function DuelResponse_MinimapButton_Reposition()
	DuelResponse_MinimapButton:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-(80*cos(DuelResponseAddonConfig.MinimapPos or DefaultMinimapPos)),(80*sin(DuelResponseAddonConfig.MinimapPos or DefaultMinimapPos))-52)
end

function DuelResponse_MinimapButton_DraggingFrame_OnUpdate()

	local xpos,ypos = GetCursorPosition()
	local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom()

	xpos = xmin-xpos/UIParent:GetScale()+70 -- get coordinates as differences from the center of the minimap
	ypos = ypos/UIParent:GetScale()-ymin-70

	DuelResponseAddonConfig.MinimapPos = math.deg(math.atan2(ypos,xpos))
	if DuelResponseAddonConfig.MinimapPos == DefaultMinimapPos then
		DuelResponseAddonConfig.MinimapPos = nil
	end
	DuelResponse_MinimapButton_Reposition() -- move the button
end

function DuelResponse_MinimapButton_OnClick()
	InterfaceOptionsFrame_OpenToCategory(_G[parent])
end

function DR_MinimapButton_OnEnter(self)
	if (self.dragging) then
		return
	end
	GameTooltip:SetOwner(self or UIParent, "ANCHOR_LEFT")
	DR_MinimapButton_Details(GameTooltip)
end


function DR_MinimapButton_Details(tt, ldb)
	tt:SetText(PlainAddonTitle)

end
